@@ -1,7 +1,7 @@
 package DBIx::Custom;
 use Object::Simple -base;
 
-our $VERSION = '0.2101';
+our $VERSION = '0.2102';
 use 5.008001;
 
 use Carp 'croak';
@@ -349,11 +349,11 @@ sub execute {
     my $sql = shift;
 
     # Options
-    my $param;
-    $param = shift if @_ % 2;
+    my $params;
+    $params = shift if @_ % 2;
     my %opt = @_;
     warn "sqlfilter option is DEPRECATED" if $opt{sqlfilter};
-    $param ||= $opt{param} || {};
+    $params ||= $opt{param} || {};
     my $tables = $opt{table} || [];
     $tables = [$tables] unless ref $tables eq 'ARRAY';
     my $filter = ref $opt{filter} eq 'ARRAY' ?
@@ -362,24 +362,25 @@ sub execute {
     # Merge second parameter
     my @cleanup;
     my $saved_param;
-    if (ref $param eq 'ARRAY') {
-        my $param2 = $param->[1];
-        $param = $param->[0];
-        for my $column (keys %$param2) {
-            if (!exists $param->{$column}) {
-                $param->{$column} = $param2->{$column};
+    if (($opt{statement} || '') ne 'insert' && ref $params eq 'ARRAY') {
+        my $params2 = $params->[1];
+        $params = $params->[0];
+        for my $column (keys %$params2) {
+            if (!exists $params->{$column}) {
+                $params->{$column} = $params2->{$column};
                 push @cleanup, $column;
             }
             else {
-                delete $param->{$_} for @cleanup;
+                delete $params->{$_} for @cleanup;
                 @cleanup = ();
-                $saved_param  = $param;
-                $param = $self->merge_param($param, $param2);
+                $saved_param  = $params;
+                $params = $self->merge_param($params, $params2);
                 delete $saved_param->{$_} for (@{$opt{cleanup} || []});
                 last;
             }
         }
     }
+    $params = [$params] unless ref $params eq 'ARRAY';
     
     # Append
     $sql .= $opt{append} if defined $opt{append} && !ref $sql;
@@ -396,8 +397,8 @@ sub execute {
         unless ($query) {
             my $c = $self->{safety_character};
             # Check unsafety keys
-            unless ((join('', keys %$param) || '') =~ /^[$c\.]+$/) {
-                for my $column (keys %$param) {
+            unless ((join('', keys %{$params->[0]}) || '') =~ /^[$c\.]+$/) {
+                for my $column (keys %{$params->[0]}) {
                     croak qq{"$column" is not safety column name } . _subname
                       unless $column =~ /^[$c\.]+$/;
                 }
@@ -413,7 +414,9 @@ sub execute {
 
     # Return query
     if ($opt{query}) {
-        delete $param->{$_} for (@cleanup, @{$opt{cleanup} || []});
+        for my $column (@cleanup, @{$opt{cleanup} || []}) {
+            delete $_->{$column} for @$params;
+        }
         return $query;
     };
     
@@ -436,8 +439,8 @@ sub execute {
            my $key = $opt{primary_key}->[$i];
            $key = "$main_table.$key" if $statement eq 'update' ||
              $statement eq 'delete' || $statement eq 'select';
-           next if exists $param->{$key};
-           $param->{$key} = $opt{id}->[$i];
+           next if exists $params->[0]->{$key};
+           $params->[0]->{$key} = $opt{id}->[$i];
            push @cleanup, $key;1
         }
     }
@@ -502,45 +505,53 @@ sub execute {
     if (!$query->{duplicate} && $type_rule_off && !keys %$filter && !$self->{default_out_filter}
       && !$opt{bind_type} && !$opt{type} && !$ENV{DBIX_CUSTOM_DEBUG})
     {
-        eval { $affected = $sth->execute(map { $param->{$_} } @{$query->{columns}}) };
-    }
-    else {
-        # Create bind values
-        my ($bind, $bind_types) = $self->_create_bind_values($param, $query->{columns},
-          $filter, $type_filters, $opt{bind_type} || $opt{type} || {});
-
-        # Execute
         eval {
-            if ($opt{bind_type} || $opt{type}) {
-                $sth->bind_param($_ + 1, $bind->[$_],
-                    $bind_types->[$_] ? $bind_types->[$_] : ())
-                  for (0 .. @$bind - 1);
-                $affected = $sth->execute;
-            }
-            else {
-                $affected = $sth->execute(@$bind);
-            }
-
-            # DEBUG message
-            if ($ENV{DBIX_CUSTOM_DEBUG}) {
-                warn "SQL:\n" . $query->{sql} . "\n";
-                my @output;
-                for my $value (@$bind) {
-                    $value = 'undef' unless defined $value;
-                    $value = encode($ENV{DBIX_CUSTOM_DEBUG_ENCODING} || 'UTF-8', $value)
-                      if utf8::is_utf8($value);
-                    push @output, $value;
-                }
-                warn "Bind values: " . join(', ', @output) . "\n\n";
+            for my $param (@$params) {
+                $affected = $sth->execute(map { $param->{$_} } @{$query->{columns}});
             }
         };
     }
+    else {
+        for my $param (@$params) {
+            # Create bind values
+            my ($bind, $bind_types) = $self->_create_bind_values($param, $query->{columns},
+              $filter, $type_filters, $opt{bind_type} || $opt{type} || {});
+
+            # Execute
+            eval {
+                if ($opt{bind_type} || $opt{type}) {
+                    $sth->bind_param($_ + 1, $bind->[$_],
+                        $bind_types->[$_] ? $bind_types->[$_] : ())
+                      for (0 .. @$bind - 1);
+                    $affected = $sth->execute;
+                }
+                else {
+                    $affected = $sth->execute(@$bind);
+                }
+
+                # DEBUG message
+                if ($ENV{DBIX_CUSTOM_DEBUG}) {
+                    warn "SQL:\n" . $query->{sql} . "\n";
+                    my @output;
+                    for my $value (@$bind) {
+                        $value = 'undef' unless defined $value;
+                        $value = encode($ENV{DBIX_CUSTOM_DEBUG_ENCODING} || 'UTF-8', $value)
+                          if utf8::is_utf8($value);
+                        push @output, $value;
+                    }
+                    warn "Bind values: " . join(', ', @output) . "\n\n";
+                }
+            };
+        }
+    }
     
     $self->_croak($@, qq{. Following SQL is executed.\n}
       . qq{$query->{sql}\n} . _subname) if $@;
 
     # Remove id from parameter
-    delete $param->{$_} for (@cleanup, @{$opt{cleanup} || []});
+    for my $column (@cleanup, @{$opt{cleanup} || []}) {
+        delete $_->{$column} for @$params;
+    }
     
     # Not select statement
     return $affected unless $sth->{NUM_OF_FIELDS};
@@ -619,19 +630,23 @@ sub insert {
     my $self = shift;
     
     # Options
-    my $param = @_ % 2 ? shift : undef;
+    my $params = @_ % 2 ? shift : undef;
     my %opt = @_;
     warn "insert method param option is DEPRECATED!" if $opt{param};
-    $param ||= delete $opt{param} || {};
+    $params ||= delete $opt{param} || {};
+    
+    my $multi;
+    if (ref $params eq 'ARRAY') { $multi = 1 }
+    else { $params = [$params] }
     
     # Timestamp(DEPRECATED!)
-    if ($opt{timestamp} && (my $insert_timestamp = $self->insert_timestamp)) {
+    if (!$multi && $opt{timestamp} && (my $insert_timestamp = $self->insert_timestamp)) {
         warn "insert timestamp option is DEPRECATED! use created_at with now attribute";
         my $columns = $insert_timestamp->[0];
         $columns = [$columns] unless ref $columns eq 'ARRAY';
         my $value = $insert_timestamp->[1];
         $value = $value->() if ref $value eq 'CODE';
-        $param->{$_} = $value for @$columns;
+        $params->[0]->{$_} = $value for @$columns;
     }
 
     # Created time and updated time
@@ -640,11 +655,11 @@ sub insert {
         my $now = $self->now;
         $now = $now->() if ref $now eq 'CODE';
         if (defined $opt{created_at}) {
-            $param->{$opt{created_at}} = $now;
+            $_->{$opt{created_at}} = $now for @$params;
             push @timestamp_cleanup, $opt{created_at};
         }
         if (defined $opt{updated_at}) {
-            $param->{$opt{updated_at}} = $now;
+            $_->{$opt{updated_at}} = $now for @$params;
             push @timestamp_cleanup, $opt{updated_at};
         }
     }
@@ -652,15 +667,15 @@ sub insert {
     # Merge id to parameter
     my @cleanup;
     my $id_param = {};
-    if (defined $opt{id}) {
+    if (defined $opt{id} && !$multi) {
         croak "insert id option must be specified with primary_key option"
           unless $opt{primary_key};
         $opt{primary_key} = [$opt{primary_key}] unless ref $opt{primary_key};
         $opt{id} = [$opt{id}] unless ref $opt{id};
         for (my $i = 0; $i < @{$opt{primary_key}}; $i++) {
            my $key = $opt{primary_key}->[$i];
-           next if exists $param->{$key};
-           $param->{$key} = $opt{id}->[$i];
+           next if exists $params->[0]->{$key};
+           $params->[0]->{$key} = $opt{id}->[$i];
            push @cleanup, $key;
         }
     }
@@ -669,15 +684,15 @@ sub insert {
     my $sql = "insert ";
     $sql .= "$opt{prefix} " if defined $opt{prefix};
     $sql .= "into " . $self->q($opt{table}) . " "
-      . $self->values_clause($param, {wrap => $opt{wrap}}) . " ";
+      . $self->values_clause($params->[0], {wrap => $opt{wrap}}) . " ";
 
     # Remove id from parameter
-    delete $param->{$_} for @cleanup;
+    delete $params->[0]->{$_} for @cleanup;
     
     # Execute query
     $opt{statement} = 'insert';
     $opt{cleanup} = \@timestamp_cleanup;
-    $self->execute($sql, $param, %opt);
+    $self->execute($sql, $params, %opt);
 }
 
 sub insert_timestamp {
@@ -2759,6 +2774,20 @@ as parameter value.
 
     {date => \"NOW()"}
 
+You can pass multiple parameters, this is very fast.
+This is EXPERIMETNAL.
+
+    $dbi->insert(
+        [
+            {title => 'Perl', author => 'Ken'},
+            {title => 'Ruby', author => 'Tom'}
+        ],
+        table  => 'book'
+    );
+
+In multiple insert, you can't use C<id> option.
+and only first parameter is used by creating sql.
+
 B<options>
 
 C<insert> method use all of C<execute> method's options,
